// Copyright (c) 2011 Paul Butcher
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

package org.scalamock

import java.lang.reflect.{InvocationHandler, Method, Proxy => JavaProxy}

private[scalamock] object Proxy {
  
  def create(classLoaderStrategy: ClassLoaderStrategy, interfaces: Class[_]*)(f: (AnyRef, Symbol, Array[AnyRef]) => AnyRef) = {
    
    val classLoader = classLoaderStrategy.getClassLoader(interfaces :_*)

    val handler = new InvocationHandler {
      def invoke(proxy: AnyRef, method: Method, args: Array[AnyRef]) =
        f(proxy, Symbol(method.getName), args)
    }

    try {
      JavaProxy.newProxyInstance(classLoader, interfaces.distinct.toArray, handler)
    } catch {
      case e: IllegalArgumentException => 
        throw new IllegalArgumentException("Unable to create proxy - possible classloader issue? Consider setting proxyClassLoaderStrategy", e)
    }
  }
}
